<?php

namespace Drupal\merlinone\Plugin\EntityBrowser\Widget;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;

/**
 * An Entity Browser widget to create media entities from MerlinOne.
 *
 * @EntityBrowserWidget(
 *   id = "merlinone_search_media_entity",
 *   label = @Translation("Media Entity MerlinOne Search"),
 *   description = @Translation("Create Media Entities from with MerlinOne library")
 * )
 */
class MediaEntityMerlinOneSearch extends MerlinOneSearch {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'media_entity_bundle' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * Returns the media bundle that this widget creates.
   *
   * @return \Drupal\media_entity\MediaBundleInterface
   *   Media bundle.
   */
  protected function getBundle() {
    return $this->entityTypeManager
      ->getStorage('media_bundle')
      ->load($this->configuration['media_entity_bundle']);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['media_entity_bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Media type'),
      '#required' => TRUE,
      '#description' => $this->t('The type of media entity to create from the uploaded file(s).'),
    ];

    $bundle = $this->getBundle();
    if ($bundle) {
      $form['media_entity_bundle']['#default_value'] = $bundle->id();
    }

    $mediaStorage = $this->entityTypeManager->getStorage('media_bundle');
    $bundles = $mediaStorage->loadByProperties(['type' => 'media_merlinone_image']);
    $bundles += $mediaStorage->loadByProperties(['type' => 'media_merlinone_document']);

    if (!empty($bundles)) {
      foreach ($bundles as $bundle) {
        $form['media_entity_bundle']['#options'][$bundle->id()] = $bundle->label();
      }
    }
    else {
      $form['media_entity_bundle']['#disabled'] = TRUE;
      $form['media_entity_bundle']['#description'] = $this->t('You must @create_bundle before using this widget.', [
        '@create_bundle' => Link::createFromRoute($this->t('create a media bundle'), 'media.bundle_add')->toString(),
      ]);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();

    // Depend on the media bundle this widget creates.
    $bundle = $this->getBundle();
    $dependencies[$bundle->getConfigDependencyKey()][] = $bundle->getConfigDependencyName();
    $dependencies['module'][] = 'media_entity';

    return $dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareEntities(array $form, FormStateInterface $form_state) {
    $entities = [];
    $bundle = $this->getBundle();

    foreach (parent::prepareEntities($form, $form_state) as $file) {
      $source_field_name = $bundle->getTypeConfiguration()['source_field'];
      $original_item = $file->original_item;

      /** @var \Drupal\file\Entity\File $file */
      // Save the file to be referenced to get an ID.
      $file->save();

      $entities[] = $this->entityTypeManager->getStorage('media')->create([
        'bundle' => $bundle->id(),
        $source_field_name => [
          'target_id' => $file->id(),
          'title' => !empty($original_item->headline) ? $original_item->headline : NULL,
        ],
        'uid' => $this->currentUser->id(),
        'status' => TRUE,
        'type' => $bundle->getType()->getPluginId(),
        // Keep original item to use when filling in mapped media fields.
        'original_item' => $original_item,
      ]);
    }

    // Keep reference to the prepared entities.
    $form_state->set('merlinone_prepared_entities', $entities);

    return $entities;
  }

  /**
   * {@inheritdoc}
   */
  public function submit(array &$element, array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\media_entity\MediaInterface[] $media_entities */
    $media_entities = $form_state->get('merlinone_prepared_entities');

    foreach ($media_entities as &$media_entity) {
      $media_entity->save();
    }

    $this->selectEntities($media_entities, $form_state);
    $this->clearFormValues($element, $form_state);
  }

}
