<?php

namespace Drupal\merlinone\Traits;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\media_entity\MediaInterface;

/**
 * A MerlinOne Media Type.
 */
trait MerlinOneMediaTypeTrait {

  use StringTranslationTrait;

  /**
   * Gets list of fields provided by Merlin.
   *
   * @return array
   *   Associative array with field names as keys and descriptions as values.
   */
  protected function getProvidedFields() {
    return [
      'headline' => $this->t('Headline'),
      'caption' => $this->t('Caption'),
      'copyright' => $this->t('Copyright'),
      'merlin_id' => $this->t('Merlin ID'),
      'keywords' => $this->t('Keywords'),
    ];
  }

  /**
   * Gets a Merlin field/value.
   *
   * @param \Drupal\media_entity\MediaInterface $media
   *   Media object.
   * @param string $name
   *   Name of field to fetch.
   *
   * @return mixed
   *   Field value or FALSE if data unavailable.
   */
  protected function getMerlinField(MediaInterface $media, $name) {
    $value = FALSE;
    $field_map = $media->bundle->entity->field_map;

    // $original_item holds the response from Merlin after entity creation,
    // so pull data from there if it exists, otherwise get the values from the
    // media entity.
    if (isset($media->original_item) && $original_item = $media->original_item) {
      switch ($name) {
        case 'merlin_id':
          $value = isset($original_item->cimageid) ? $original_item->cimageid : FALSE;
          break;

        case 'caption':
          $value = isset($original_item->capt2120) ? $original_item->capt2120 : FALSE;
          break;

        case 'keywords':
          if (!empty($original_item->keywords)) {
            /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
            $term_storage = $this->getEntityTypeManager()->getStorage('taxonomy_term');
            $value = [];

            $handler_settings = $media->getFieldDefinition($field_map[$name])->getSetting('handler_settings');
            $keywords_vocabulary = count($handler_settings['target_bundles']) > 1 ? $handler_settings['auto_create_bundle'] : reset($handler_settings['target_bundles']);

            foreach (explode(',', $original_item->keywords) as $keyword) {
              $keyword = trim($keyword);
              $term_candidates = $term_storage->loadByProperties([
                'vid' => $keywords_vocabulary,
                'name'  => $keyword,
              ]);

              if (empty($term_candidates)) {
                $term = $term_storage->create([
                  'vid' => $keywords_vocabulary,
                  'name' => $keyword,
                ]);
                $term->save();
              }
              else {
                $term = array_shift($term_candidates);
              }

              $value[] = $term->id();
            }
          }
          break;

        default:
          $value = isset($original_item->{$name}) ? $original_item->{$name} : FALSE;
      }
    }
    else {
      $media_values = $media->toArray();
      if (isset($media_values[$field_map[$name]])) {
        $value = $media_values[$field_map[$name]];
      }
    }

    return $value;
  }

  /**
   * Get the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager service.
   */
  abstract protected function getEntityTypeManager();

}
